#!/usr/bin/perl
#**********************************************************************#
# Weblog v1.6 by Jonathan Eisenzopf <eisen@xif.com>                    #
# Copyright (c) 2001 XIF Communications. All Rights Reserved.          #
#**********************************************************************#
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#**********************************************************************#
#
#**********************************************************************#
# INCLUDES - I wouldn't recommend changing this                        #
#**********************************************************************#
use strict;
use CGI qw/:standard *table :cgi-lib/;
use CGI::Carp qw(fatalsToBrowser);
#use CGI::Carp;
use XML::RSS;
use XML::Simple;
use HTML::Template;
use HTML::Entities;
use Fcntl qw^:DEFAULT :flock^;
use File::Copy;
use POSIX qw(strftime);


# Config Stuff
BEGIN {
use vars qw( %channel $config $current_channel $sysadmin_err
	$err_header $err_trail  $move @cgi_key_names $current_channel_rss $current_rss $d ); 
}

# Death Debug Dumper

#my $cur_dir = `pwd`;
#chomp($cur_dir);
my $config_file = "../config/config.xml";
my $sysadmin_err = "Please contact your Systems Administrator and give him the error message below:";
my $err_header =  "<h2>";
my $err_trail = "</h2>";

# Load up the XML Config stuff
&load_Config();

#**********************************************************************#
# MAIN body if the code                                                #
#**********************************************************************#

### Current Channel Check
my $current_channel = param('channel');
my $current_rss = param('rss');

if(!defined($current_channel)) {
	$current_channel = $config->{default_channel};
	&InitError('default_channel') if(!defined($config->{default_channel}));
}

# Now grab them by name
my %channel = ();
for my $k ( @{ $config->{channel} } ) {
		$channel{ $k->{name} } = $k;
}


# Do we even have an RSS to read?
if(!defined($channel{$current_channel}->{$current_rss})) {
		$current_rss = 'rss1.0';
}

# Hmmmm....not liking this output :P
if(!defined($channel{$current_channel}->{$current_rss})) {
	&InitError("$current_channel rss1.0");
}

$current_channel_rss = $channel{$current_channel}->{$current_rss};
my @cgi_key_names = param();
#warn("cgi_key_names are @cgi_key_names");
my $move = undef;
for(@cgi_key_names) {
	if ($_ =~ /ITEM/) {
		$move++;	
	}
}

my $rss = new XML::Parser;
if (param('ACTION') eq 'Build RSS') {
    #print "Location: $config->{script_url}\n\n";
	my $rss = &generate_RSS($current_channel);
    &build_HTML($rss,$current_channel);
    &build_palm($rss,$current_channel);
    &build_wml($rss,$current_channel);
    Delete_all;
	print header;
	&parse_rss($current_channel_rss);
    &print_form;
} elsif(defined($move)) {
	my $rss = &generate_RSS($current_channel,'move');
    &build_HTML($rss,$current_channel);
    &build_palm($rss,$current_channel);
    &build_wml($rss,$current_channel);
    Delete_all;
	print header;
	&parse_rss($current_channel_rss);
    &print_form;
# Old Way
#	print header;
#	my $rss = &parse_rss($current_channel_rss,param('direction'));
#	&print_form;
} elsif (@ARGV > 0) {
    my $rss = &generate_RSS;
    &build_HTML($rss,$current_channel);
    &build_palm($rss,$current_channel);
    &build_wml($rss,$current_channel);
    print "Done.\n";

} else {
    Delete_all;
    print header;
    &parse_rss($current_channel_rss);
    &print_form;
}
#**********************************************************************#
# SUBROUTINES                                                          #
#**********************************************************************#

# Init Error
# Something we needed wasnt defined
# Think about hard-coding these values???
sub InitError {
	my $key = shift;
	print header, 
		start_html,
		h2("WebLog configuration Error: $key must be defined in your conifg.xml file for weblog to run"),
	 	end_html;
	exit;
}


#**********************************************************************#
# build_HTML() creates the HTML output for the RSS file and dumps it   #
# in the story/html directory                                          #
#**********************************************************************#
sub build_HTML {
    my ($rss,$curchan)  = @_;

	return if(!defined($channel{$curchan}->{html}));

	my $tmpl = $channel{$curchan}->{tmpl_dir} . "/" .  $config->{templates}->{html}; 
    my $template = HTML::Template->new(filename => $tmpl);
    

	open(HTML,">$channel{$curchan}->{html}") ||
		croak("$err_header Cannot write to $channel{$curchan}->{html} $err_trail $!");

	my @tmpl_array = ();

    # build the data structure for HTML::Template
    my $count = 1;
    foreach my $item (@{$rss->{'items'}}) {
		next unless $item->{'title'} && $item->{'link'};
		last unless $count <= $channel{$curchan}->{max_items}->{'html'};
		my %row = ();
		$row{'LINK'} = $item->{'link'};
		$row{'TITLE'} = $item->{'title'};
		$row{'DESCRIPTION'} = $item->{'description'};
		push(@tmpl_array,\%row);
		$count++;
    }

    # set item array in template
    $template->param(RSS_LOOP => \@tmpl_array);

    # set title
    $template->param(FULLNAME => $rss->channel('title'));

    # generate the HTML file
    print HTML $template->output();
    close(HTML);
}

#**********************************************************************#
# build_palm builds a simple HTML version of the RSS file mean for Palm#
# and mobile devices. It conforms to the Palm PQA HTML formats.        #
#**********************************************************************#
sub build_palm {
    my ($rss, $curchan) = @_;
	return if(!defined($channel{$curchan}->{palm}));

	my $palm_tmpl = $channel{$curchan}->{tmpl_dir} . "/" . $config->{templates}->{palm};
    my $template = HTML::Template->new(filename => $palm_tmpl);
 
 	open(HTML,">$channel{$curchan}->{palm}") || 
		croak("$err_header Cannot write to $channel{$curchan}->{palm} $err_trail $!");

    my @index_array = ();

    # build the data structure for index file
    my $count = 1;
    foreach my $item (@{$rss->{'items'}}) {
		next unless $item->{'title'} && $item->{'link'};
		last unless $count <= $channel{$curchan}->{max_items}->{'palm'};
		my %row = ();
		$row{'LINK'} = "$channel{$curchan}->{palm_url}/$count.html";
		$row{'TITLE'} = &concat_text($channel{$curchan}->{palm_title_max},$item->{'title'});
		
		push(@index_array,\%row);
		# build story file
		my $story_tmpl = $channel{$curchan}->{tmpl_dir} . "/" . $config->{templates}->{palm_story};
		my $story = HTML::Template->new(filename => $story_tmpl);

		open(STORY,">$channel{$curchan}->{palm_story}/$count.html") || 
			croak("$err_header Cannot write to $channel{$curchan}->{palm_story}/$count.html $err_trail");

		$story->param(STORY_TITLE => $item->{'title'});
		$story->param(DESCRIPTION => &concat_text($channel{$curchan}->{palm_desc_max},$item->{'description'}));
		$story->param(LINK        => $item->{'link'});

		# print the story file
		print STORY $story->output();
		close(STORY);

		# increment story counter
		$count++;
    }

    # set item array in template
    $template->param(RSS_LOOP => \@index_array);

    # set title
    $template->param(FULLNAME => $rss->channel('title'));

    # generate the Palm HTML index file
    print HTML $template->output();
    close(HTML);
}
#**********************************************************************#
# build_wml() builds the WML text from the RSS file. Care should be    #
# taken not to put too much text in the items or it will exceed the    #
# storage space of some phones.                                        #
#**********************************************************************#
sub build_wml {
    my ($rss,$curchan) = @_;

	return if(!defined($channel{$curchan}->{wml}));

	my $wml_tmpl = $channel{$curchan}->{tmpl_dir} . "/" . $config->{templates}->{wml};

    my $template = HTML::Template->new(filename => $wml_tmpl);
 	open(HTML,">$channel{$curchan}->{wml}") || 
		croak("$err_header Cannot write to $channel{$curchan}->{wml} $err_trail");


	my @index_array = ();

    # build the data structure for index file
    my $count = 1;
    foreach my $item (@{$rss->{'items'}}) {
		next unless $item->{'title'} && $item->{'link'};
		last unless $count <= $channel{$curchan}->{max_items}->{'wml'};
		my %row = ();
		$row{'LINK'}  = "$channel{$curchan}->{wml_url}/$count.wml"; 
		$row{'TITLE'} = &concat_text($channel{$curchan}->{wml_title_max},$item->{'title'});
		push(@index_array,\%row);

		# build story file
		my $story_tmpl = $channel{$curchan}->{tmpl_dir} . "/" . $config->{templates}->{wml_story};
		my $story = HTML::Template->new(filename => $story_tmpl);

		open(STORY,">$channel{$curchan}->{wml_story}/$count.wml") ||
				croak(" $err_header Cannot write to $channel{$curchan}->{wml_story}/$count.wml $err_trail $!");

		$story->param(DESCRIPTION => &concat_text($channel{$curchan}->{wml_desc_max},$item->{'description'}));
		$story->param(COUNTER     => $count);
		$story->param(LINK        => $item->{'link'});
		$story->param(STORY_TITLE => $item->{'title'});

		# print the story file
		print STORY $story->output();
		close(STORY);

		# increment story counter
		$count++;
    }

    # set item array in template
    $template->param(RSS_LOOP => \@index_array);

    # set title
    $template->param(FULLNAME => $rss->channel('title'));

    # generate the Palm HTML index file
    print HTML $template->output();
    close(HTML);
}

sub concat_text {
#**********************************************************************#
# concat_text() concatenates text that is too long. Used for Palm      #
# and WML outputs.                                                     #
#**********************************************************************#
    my ($maxchars, @wordstack);
    
    $maxchars = shift;
    my $string = shift;
    if (length($string) > $maxchars) {
	$string = substr($string,0,($maxchars - 2));
	$string .= "..";
    }
    
    $string;
}

#**********************************************************************#
# encode() encodes default XML entities.                               #
#**********************************************************************#
sub encode {
    my $text = shift;
    $text = HTML::Entities::decode($text);
    $text =~s/\r//gsi;        # delete carraige returns - abk 0926
    $text =~s/\s*\n\s*/ /gsi; # replace returns w/ space
    $text =~ s/[^\w\s\\\/\<\>\&\@\.\,\:\;\'\"\{\}\[\]\-\_\+\=\(\)\!\~\`\@\#\$\%\^\*]//g;
    $text = HTML::Entities::encode($text, '\200-\377\'\%\&\<\>\"');
    return $text;
}


#**********************************************************************#
# generate_RSS() updates the RSS file based on user input on the Web   #
# form.                                                                #
#**********************************************************************#
sub generate_RSS {
    my ($curchan,$move) = @_;
    my $dc = {};
    
    # create instance of XML::RSS
    # let's start with this and switch the output below
    my $rss = new XML::RSS(version => '1.0');
    
    # Do we even want any RSS output
    if(!defined($channel{$curchan}->{'rss1.0'}) && 
       !defined($channel{$curchan}->{'rss0.9'}) &&
       !defined($channel{$curchan}->{'rss0.91'})) {
	return $rss; # return empty rss object
    }


    # channel
    $rss->channel(
                  'title'       => param('ctitle'),
                  'link'        => param('clink'),
                  'description' => param('cdesc'),
		  'language'    => $config->{'language'}
                  );

    $dc->{'date'     } = strftime("%Y-%m-%dT%H:%M",gmtime);
    $dc->{'publisher'} = &encode(param('cpublisher')) if param('cpublisher');
    $dc->{'creator'  } = &encode(param('ccreator'  )) if param('ccreator');
    $dc->{'rights'   } = &encode(param('crights'   )) if param('crights');
    $dc->{'subject'  } = &encode(param('csubject'  )) if param('csubject');
    $rss->channel(dc => $dc);

    # image
    if (param('iurl') =~ /\S+/) {
        $rss->image(
                    'title' => &encode(param('ititle')),
                    'url'   => &encode(param('iurl')),
                    'link'  => &encode(param('ilink'))
                    );
	$rss->image('description' => param('idesc')) if param('idesc');
    }
    
    # textinput
    if (param('tilink')) {
        $rss->textinput(
                        'title'       => &encode(param('tititle')),
                        'description' => &encode(param('tidesc')),
                        'name'        => &encode(param('tiname')),
                        'link'        => &encode(param('tilink'))
                        );
    }
    

    # items
    for (my $i=1; $i <= 15; $i++) {
	next if defined(param("delete" . $i));

	if (param("item".$i."_title") && param("item".$i."_link")
	    && param("item".$i."_desc")) {
	    $rss->add_item(
			   'title'        => &encode(param("item".$i."_title")),
			   'link'         => &encode(param("item".$i."_link")),
			   'description'  => &encode(param("item".$i."_desc")),
			   dc => {
			       subject  => &encode(param("item".$i."_subject")),
			       creator  => &encode(param("item".$i."_creator")),
			       date     => &encode(param("item".$i."_date"))
			   }
			   );
	}
    }

    
    # new item 1
    if (param('newtitle1') && param('newlink1') && param('newdesc1')) {
	pop(@{$rss->{'items'}}) if (@{$rss->{'items'}} >= 15);
	$rss->add_item(
		       'title' => &encode(param('newtitle1')),
		       'link'  => &encode(param('newlink1')),
		       'description' => &encode(param('newdesc1')),
		       'mode'  => 'insert',
		       dc => {
			       subject  => &encode(param('newsubject1')),
			       creator  => &encode(param('newcreator1')),
			       date     => &encode(strftime("%Y-%m-%dT%H:%M",gmtime))
			   }
		       );
    }
    
    # Shuffle the items.
    # If moving the item around
    if(defined($move)) {
	my $p = undef;
	my $cgi_item = undef;
	my $direc = undef;
	# need to have key here to find the item
	for my $cgi_i (@cgi_key_names) {
	    if($cgi_i =~ /ITEM/) {
		($cgi_item,$p) = split /ITEM/, $cgi_i;
		$direc = param($cgi_i);
	    }
	}
	if(defined($p) && defined($direc)) {
	    $p--;
	    if($direc =~ /down/i) {
		($rss->{'items'}->[$p+1], $rss->{'items'}->[$p]) = 
		    ($rss->{'items'}->[$p], $rss->{'items'}->[$p+1]);
	    } else {
		($rss->{'items'}->[$p], $rss->{'items'}->[$p-1]) =
		    ($rss->{'items'}->[$p-1], $rss->{'items'}->[$p]);
	    }
	}
    }
    
    for my $output_type ('0.9', '0.91', '1.0') {
	my $frss = 'rss' . $output_type;
	if(defined($channel{$curchan}->{$frss})) {
	    $rss->{output} = $output_type;
	    
	    # Archive first...do NOT open/lock the file first
	    # Let's be super paranoid and obtail a lock first
	    &obtain_lock();		
	    &Archive($channel{$curchan}->{$frss});
	    &release_lock();
	    
	    &obtain_lock();
	    my $tmp_rss = &TempRss($rss);
	    &CheckTempRss($tmp_rss);
	    &release_lock();	
	    
	    &obtain_lock();
	    # save the RSS file
	    # there is a croak in XML::RSS::save but let's check here also
	    eval {
		$rss->save($channel{$curchan}->{$frss});
	    };
	    
	    if($@) {
		croak ("$err_header There was an error saveing your RSS file $sysadmin_err $err_trail  $@\n");
	    }	
	    
	    &release_lock();
	}
    }
    return $rss;
}


sub TempRss {
    my $rss = shift;
    &InitError("temp_dir") unless (-d $config->{temp_dir});
    open(TMPRSS, "+> $config->{temp_dir}/rss_tmp") or croak "$err_header Cannot open a temp file for testing rss $err_trail  $sysadmin_err $!\n";
    eval {
	$rss->save("$config->{temp_dir}/rss_tmp")
	};
    if($@) {
	croak ("$err_header XML::RSS had some errors saving the tmp file $sysadmin_err $err_trail  $@\n");
    } else {
	return("$config->{temp_dir}/rss_tmp");
    }
}


sub CheckTempRss {
    my $file = shift;
    my $check_rss = XML::RSS->new();
    
    eval {
	$check_rss->parsefile($file);
    };
    
    if($@) {
	croak ("$err_header There was an Error in building your  RSS file $sysadmin_err $err_trail   $@\n");	
    } else {
	unlink $file or croak ("$err_header Cannot unlink $file $sysadmin_err $err_trail $@\n");
	return 1;
    }   
}


###########################
# LOCK  Controls          #
###########################
sub obtain_lock {
    croak ("$err_header lock dir: $config->{lock_dir} does not exist $err_trail \n") unless -d $config->{lock_dir};
    my $lock_counter = 0;
    
  LOCKIT:open(LOCKED, "+> $config->{lock_dir}/lock") or 
      croak ("$err_header Cannot grok opening $config->{lock_dir}/lock $err_trail $!\n");
    if(!flock(LOCKED, LOCK_EX)) {
	sleep 1;
	$lock_counter++;
	close(LOCKED) or croak ("$err_header Cannot grok closing $config->{lock_dir}/lock $err_trail $!\n");
	$config->{lock_retries} < $lock_counter ? goto LOCKIT : croak ("$err_header Cannot grab an exclusive lock! $err_trail \n");
    }	
}


sub release_lock {
    close(LOCKED) or croak ("$err_header Cannot grok closing $config->{lock_dir}/lock $err_trail $!\n");
    unlink("$config->{lock_dir}/lock") or croak ("$err_header Cannot grok unlinking $config->{lock_dir}/lock $err_trail $!\n");
    
}


sub Archive {
    croak ("$err_header Archive directory does not exist $config->{archive_dir} $err_trail  \n") unless -d $config->{archive_dir};
    my $dst = &date_time_stamp();
    my $fname = (split(/\//, $current_channel_rss))[-1];
    my $dated_copy = $fname . "-" . $dst;
    
    unless(copy($current_channel_rss, "$config->{archive_dir}/$dated_copy")) {
	croak ("$err_header Cannot copy $current_channel_rss to $config->{archive_dir}/$dated_copy $err_trail  $!\n");
    }
    
}


sub date_time_stamp {
    my($year,$month,$day,$hour,$min,$sec ) = (localtime)[5,4,3,2,1,0];
    $year = $year + 1900;
    my $dst = $year . $month . $day . $hour . $min . $sec;
    return $dst;
}


sub load_Config {
    $config = XMLin($config_file, keyattr=>["channel"], forcearray => ["channel"]);
}


#**********************************************************************#
# parse_rss() parses the RSS source file and puts the values into      #
# internal values that are dumped into the Web form.                   #
#**********************************************************************#
sub parse_rss {
    my ($file,$direc) = @_;
	
	if(! -e $file) {
		croak ("$err_header RSS File ( $file ) does not exist $err_trail \n");	
	}
	
	my $rss = new XML::RSS;
    eval { 
	$rss->parsefile($file) 
    };
   
    if ($@) {
	print "$err_header There was an Error parsing the RSS file ($file) $err_trail : It's possible you have encoded html stings\n";
	print "in your RSS file, especially if you did not create the rss file with weblog: $sysadmin_err $@\n";
	exit;
    }

    # set channel params
    param(-name=>'ctitle',     -value=>$rss->channel('title'));
    param(-name=>'cdesc',      -value=>$rss->channel('description'));
    param(-name=>'clink',      -value=>$rss->channel('link'));
    param(-name=>'cpublisher', -value=>$rss->{channel}->{dc}->{publisher});
    param(-name=>'ccreator',   -value=>$rss->{channel}->{dc}->{creator});
    param(-name=>'crights',    -value=>$rss->{channel}->{dc}->{rights});
    param(-name=>'cdate',      -value=>$rss->{channel}->{dc}->{date});
    param(-name=>'csubject',   -value=>$rss->{channel}->{dc}->{subject});

    # set image params
    param(-name=>'ititle',  -value=>$rss->image('title'));
    param(-name=>'iurl',    -value=>$rss->image('url'));
    param(-name=>'ilink',   -value=>$rss->image('link'));
    param(-name=>'iwidth',  -value=>$rss->image('width'));
    param(-name=>'iheight', -value=>$rss->image('height'));
    param(-name=>'idesc',   -value=>$rss->image('description'));

    # set text input params
    param(-name=>'tititle', -value=>$rss->textinput('title'));
    param(-name=>'tidesc',  -value=>$rss->textinput('description'));
    param(-name=>'tiname',  -value=>$rss->textinput('name'));
    param(-name=>'tilink',  -value=>$rss->textinput('link'));


    # set item params
    for (my $i=0; $i < @{$rss->{'items'}}; $i++) {
        my $item = $rss->{'items'}->[$i];
        param(-name=>"item".($i+1)."_title", -value=>$item->{'title'});
        param(-name=>"item".($i+1)."_link", -value=>$item->{'link'});
	param(-name=>"item".($i+1)."_desc", -value=>$item->{'description'});
	param(-name=>"item".($i+1)."_creator", -value=>$item->{dc}->{'creator'});
	param(-name=>"item".($i+1)."_subject", -value=>$item->{dc}->{'subject'});
	param(-name=>"item".($i+1)."_date", -value=>$item->{dc}->{'date'});
    }
}


#**********************************************************************#
# print_form() prints the Web form that's used to manage the RSS       #
# channel.                                                             #
#**********************************************************************#
sub print_form {
    my $ctitle     = param('ctitle');
    my $cdesc      = param('cdesc');
    my $clink      = param('clink');
    my $cpublisher = param('cpublisher');
    my $ccreator   = param('ccreator');
    my $crights    = param('crights');
    my $csubject   = param('csubject');
    my $ititle     = param('ititle');
    my $iurl       = param('iurl');
    my $ilink      = param('ilink');
    my $iwidth     = param('iwidth');
    my $iheight    = param('iheight');
    my $idesc      = param('idesc');
    my $tititle    = param('tititle');
    my $tidesc     = param('tidesc');
    my $tiname     = param('tiname');
    my $tilink     = param('tilink');
    my $rss_url    = param('rss_url');

    print start_html(-title=>'Weblog Manager',
		     -BGCOLOR=>'#FFFFFF');

    print h1({-align=>'CENTER'},'Weblog Manager');

    print startform(-method=>'POST',
                    -action=>url());
	print hidden(-name=>'channel', -value=>"$current_channel");

 #   print table({-border=>undef -align=>'center'});
    # Colors.......
    my $green = '#80FF80';
    my $blue  = '#8080FF';
    my $grey  = '#CCCCCC';
    
    # Legend
    print "<p align=center> <b>Legend:</b> ";
    my $legend = "<font color=$green>New Item</font> | ";
    $legend .= "<font color=$grey>  Existing Item</font> | ";
    $legend .= "<font color=$blue> Channel Info </font>\n"; 
    print b($legend);
    print "</p>\n";
    
    
    # Channel Selector -- a little ugly :P
    my @chans =  keys %channel;
    my $num_chans = scalar(@chans);
    
    print "<p align=center> <b>RSS Channels:</b> ";	
    for my $cl (@chans) {
	my $c = $cl;
	if($cl eq $current_channel) {
	    $c = "<font color=\"#FF0000\">$c</font>";
	}
	if($cl ne $current_channel ) {
	    if($cl ne $chans[-1] && $num_chans > 1) {
		print a({href=>"weblog.pl?channel=$cl"}, " $c "), '|';
	    } else {
		print a({href=>"weblog.pl?channel=$cl"}, " $c ");
	    }
	} else {
	    print " $c ", ' | ';
	}
    }	
    print "</p>\n";
    
    # Super Table 
    print table({-border=>1, -width=>'100%',-align=>'center', -cellspacing=>1, -cellpadding=>3});
    # New Item Table
    print table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3,bgcolor=>$green});
    
    # new channel item
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
	     [
	      th({-colspan=>3,-BGCOLOR=>'#000000'}, 
		 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},"New Item ")
		 ),
	      
	          td({-align=>'right'},[b('Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_title">?</a>):'),
                     textfield(-name=>'newtitle1',
                               -size=>50,
			       -default=>''
                               ),
                     submit(-name=>'ACTION',-value=>'Build RSS')
                     ]),

                  td({-align=>'right'},[b('Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_link">?</a>):'),
                     textfield(-name=>'newlink1',
                               -size=>50,
			       -default=>''
                               ),
                     ]),

		  td({-align=>'right'},[b('Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_desc">?</a>):'),
                     textarea(-name=>'newdesc1',			      
			      -columns=>50,
			      -rows=>5,
			      -default=>''
                               ),

                     ]),

	          td({-align=>'right'},[b('Creator:'),
                     textfield(-name=>'newcreator1',
                               -size=>50,
			       -default=>''
                               ),
                     ]),

	          td({-align=>'right'},[b('Subject:'),
                     textfield(-name=>'newsubject1',
                               -size=>50,
			       -default=>''
                               ),
                     ]),
              ]);
	# Creates the space between the new and the old items
	#print Tr({-align=>'LEFT',-valign=>'MIDDLE'},[th({-colspan=>3}),td(),td(),td(),]);
	print "</table>\n";
	print "</table>\n";
	print "<p><br></p>\n";
	# End of new item table

	# Re-instantiate Super Table
	print table({-border=>1, -width=>'100%', -align=>'center', -cellspacing=>1, -cellpadding=>3});

	print table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3, bgcolor=>$grey});
 
    # channel items
    for (my $i=1; $i<=15; $i++) {
        my %title;
        my %link;
	my %desc;
	my %creator;
	my %subject;
	my %date;
        $title{'name'}    = "item".$i."_title";
        $title{'value'}   = param($title{'title'});
	$desc{'name'}     = "item".$i."_desc";
	$desc{'value'}    = param($desc{'name'});
        $link{'name'}     = "item".$i."_link";
        $link{'value'}    = param($link{'name'});
	$creator{'name'}  = "item".$i."_creator";
	$creator{'value'} = param($creator{'name'});
	$subject{'name'}  = "item".$i."_subject";
	$subject{'value'} = param($subject{'name'});
	$date{'name'}  = "item".$i."_date";
	$date{'value'} = param($date{'name'});

	my $ucode;

		if($i != 1 &&  $i !=15) {
			$ucode = "table({-align=>'left'},td({-rowspan=>2},submit(-name=>\"ITEM$i\" , -value=>\" up    \"), br(),  submit(-name=>\"ITEM$i\", -value=>\"down\")))";
		
		
		    } else {
			$ucode = undef;
		    }

        print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
                 [
                  th({-colspan=>3,-BGCOLOR=>'#000000'},

font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},"Item$i")
                     ),

                  td({-align=>'right'},[b('Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_title">?</a>):'),
                     textfield(-name=>$title{'name'},
                               -default=>$title{'value'},
                               -size=>50
                               ),
                     submit(-name=>'ACTION',-value=>'Build RSS')
                     ]),

                  td({-align=>'right'},[b('Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_link">?</a>):'),
                     textfield(-name=>$link{'name'},
                               -default=>$link{'value'},
                               -size=>50
                               ),
		     checkbox(-name=> "delete" . $i, -label => ' Delete')
                     ]),

		  td({-align=>'right'},[b('Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#item_desc">?</a>):'),
                     textarea(-name=>$desc{'name'},
			      -default=>$desc{'value'},
			      -columns=>50,
			      -rows=>5
                               ),
		     eval ($ucode),
		     ]),

		  td({-align=>'right'},[b('Creator:'),
                     textfield(-name=>$creator{'name'},
                               -size=>50,
			       -default=>$creator{'value'}
                               ),
                     ]),

	          td({-align=>'right'},[b('Subject:'),
                     textfield(-name=>$subject{'name'},
                               -size=>50,
			       -default=>$subject{'value'}
                               ),
                     ]),

		  td({-align=>'right'},[b('Date:'),
                     textfield(-name=>$date{'name'},
                               -size=>50,
			       -default=>$date{'value'}
                               ),
                     ]),
		  ]);
    }
   print "</table>\n";
   print "</table>\n";
   print "<p><br></p>\n";

	# Summary Section
	print table({-border=>1, -width=>'100%',-align=>'center', -cellspacing=>1, -cellpadding=>3});

   print table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3,bgcolor=>$blue});

    # channel summary
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Channel Summary')
                 ),
              td({-align=>'right'},[b('Channel Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_title">?</a>):'),
                  textfield(-name=>'ctitle',
                            -default=>$ctitle,
                            -size=>50
                            )
                  ]),

              td({-align=>'right'},[b('Channel Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_link">?</a>):'),
                  textfield(-name=>'clink',
                           -default=>$clink,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Description(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_description">?</a>):'),
                  textarea(-name=>'cdesc',
                           -default=>$cdesc,
                           -rows=>5,
                           -columns=>50
                           )
                  ]),


	      td({-align=>'right'},[b('Publisher(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_editor">?</a>):'),
                  textfield(-name=>'cpublisher',
                           -default=>$cpublisher,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Editor(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_webmaster">?</a>):'),
                  textfield(-name=>'ccreator',
                           -default=>$ccreator,
                           -size=>50
                           )
                  ]),


	      td({-align=>'right'},[b('Copyright(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_copyright">?</a>):'),
                  textfield(-name=>'crights',
                           -default=>$crights,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Subject(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#channel_lastbuild">?</a>):'),
                  textfield(-name=>'csubject',
                           -default=>$csubject,
                           -size=>50
                           )
                  ])
              ]);

    # channel image
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Channel Image')
                 ),

              td({-align=>'right'},[b('Image Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_title">?</a>):'),
                  textfield(-name=>'ititle',
                           -default=>$ititle,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Image URL(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_url">?</a>):'),
                  textfield(-name=>'iurl',
                           -default=>$iurl,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Image Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_link">?</a>):'),
                  textfield(-name=>'ilink',
                           -default=>$ilink,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Width(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_width">?</a>):'),
                  textfield(-name=>'iwidth',
                           -default=>$iwidth,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Height(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_height">?</a>):'),
                  textfield(-name=>'iheight',
                           -default=>$iheight,
                           -size=>50
                           )
                  ]),

	      td({-align=>'right'},[b('Image Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#image_desc">?</a>):'),
                  textarea(-name=>'idesc',
                           -default=>$idesc,
                           -columns=>50,
			   -rows=>5
                           )
                  ])
              ]);


    # channel textarea
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              th({-colspan=>2,-BGCOLOR=>'#000000'},
                 font({-color=>'#FFFFFF',-font=>'Arial,Helvetica',-size=>'+1'},
                      'Text Input')
                 ),

              td({-align=>'right'},[b('Input Title(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_title">?</a>):'),
                  textfield(-name=>'tititle',
                           -default=>$tititle,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Name(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_name">?</a>):'),
                  textfield(-name=>'tiname',
                           -default=>$tiname,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Link(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_link">?</a>):'),
                  textfield(-name=>'tilink',
                           -default=>$tilink,
                           -size=>50
                           )
                  ]),

              td({-align=>'right'},[b('Input Desc(<a target="_new" href="http://www.webreference.com/perl/tools/makerss.html#input_description">?</a>):'),
                  textarea(-name=>'tidesc',
                           -default=>$tidesc,
                           -columns=>50,
                           -rows=>5
                           )
                  ])
              ]);

    print "</table>\n";
    print '</TABLE>';
    print end_form;
}
#### END

